package com.tecnavia.tabridge;

import android.content.res.Configuration;
import android.os.Bundle;
import android.os.LocaleList;
import android.view.ActionMode;
import android.view.View;
import android.view.WindowManager;

import androidx.appcompat.app.AppCompatActivity;

import com.facebook.react.modules.core.DefaultHardwareBackBtnHandler;
import com.tecnavia.tabridge.hacks.ChartbeatReflection;
import com.tecnavia.tabridge.utils.TaLang;

import java.util.Locale;

public class TaActivity extends AppCompatActivity implements DefaultHardwareBackBtnHandler {
	private boolean activeModeVisible = false;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		setLayoutDirection(getLocale());
		setLocaleContext(getLocale());
		super.onCreate(savedInstanceState);
	}

	public String getLocale() {
		return Locale.ENGLISH.getLanguage();
	}

	@Override
	public void onActionModeStarted(ActionMode mode) {
		super.onActionModeStarted(mode);
		activeModeVisible = true;
	}

	@Override
	public void onActionModeFinished(ActionMode mode) {
		super.onActionModeFinished(mode);
		activeModeVisible = false;
	}

	/**
	 * apply language to application configuration
	 * @param defaultLocale locale string
	 */
	private void setLocaleContext(String defaultLocale){
		try {
			Locale locale = TaLang.getLanguage(defaultLocale);
			Locale.setDefault(locale);
			Configuration config = new Configuration();
			config.setLocale(locale);

			LocaleList localeList = new LocaleList(locale);
			config.setLocales(localeList);
			getResources().updateConfiguration(config, getResources().getDisplayMetrics());
		} catch (NullPointerException npe){
			npe.printStackTrace();
		}
	}

	/**
	 * on old android there was a problem because the window is always resized and caused a complete redraw of application
	 * with adjust nothing the content not scroll automatically to focused item but there aren't any restart
	 */
	protected void setSoftInputMode(){
		try {
			getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_PAN);
		} catch(Exception e){
			e.printStackTrace();
		}
	}

	/**
	 * set direction off application layout LTR or RTL
	 * @param locale language string
	 */
	protected void setLayoutDirection(String locale){
		try {
			if (TaLang.isRTL(locale)) {
				getWindow().getDecorView().setLayoutDirection(View.LAYOUT_DIRECTION_RTL);
			} else {
				getWindow().getDecorView().setLayoutDirection(View.LAYOUT_DIRECTION_LTR);
			}
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	@Override
	public void invokeDefaultOnBackPressed() {
		finish();
	}

	public boolean isActiveModeVisible() {
		return activeModeVisible;
	}
}
