package com.tecnavia.tabridge;

import android.content.Context;
import android.os.Build;
import android.webkit.WebView;

import com.facebook.react.ReactInstanceManager;
import com.facebook.react.modules.i18nmanager.I18nUtil;
import com.tecnavia.tabridge.utils.TaReflection;
import com.tecnavia.tabridge.utils.TaUtils;

public class TaContext {

	public static void init(Context context){
		if(BuildConfig.DEBUG) {
			setDebugContext();
		}
		setWebViewContext(context);
		setLayoutContext(context);
	}

	/**
	 * must call inside Application onCreate
	 */
	public static void initFlipper(Context context, ReactInstanceManager reactInstanceManager){
		if (!TaUtils.isKindle()) {
			TaReflection.initializeFlipper(context, reactInstanceManager);
		}
	}

	/**
	 * initialize flipper, webview debugging and uncaught exception handler
	 */
	private static void setDebugContext(){
		try {
			Thread.setDefaultUncaughtExceptionHandler((t, e) -> e.printStackTrace());
			WebView.setWebContentsDebuggingEnabled(true);
		} catch (NullPointerException npe){
			npe.printStackTrace();
		}
	}


	/**
	 * set directory suffix, requested from android P
	 * suffix will be the process name
	 */
	private static void setWebViewContext(Context context){
		try {
			if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
				String packageName = context.getPackageName();
				String processName = TaUtils.getProcessName(context);
				if (!packageName.equals(processName)) {
					WebView.setDataDirectorySuffix(processName);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * enable react native RTL support
	 */
	private static void setLayoutContext(Context context){
		try {
			I18nUtil sharedI18nUtilInstance = I18nUtil.getInstance();
			sharedI18nUtilInstance.allowRTL(context, true);
		} catch (Exception e){
			e.printStackTrace();
		}
	}
}
