package com.tecnavia.tabridge;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import java.util.Arrays;

import com.tecnavia.tabridge.utils.TaUtils;

/**
 * the class handle props and intent and return a single Bundle for react native
 */
class TaProps {
	private static final String TAG = "TA_INTENT_UTILS";
	private final Context mContext;

	private Bundle buildProps;
	private Bundle optionalProps;

	public TaProps(Context context) {
		mContext = context;
	}

	/**
	 * add to base props the value from intent
	 */
	public Bundle getPropsFromIntent(Intent intent) throws Exception {
		Bundle props = getProps();
		Bundle intentProps = new Bundle();
		if(intentNotEmpty(intent)){
			intentProps = getBundleFromIntent(intent);
			props.putAll(intentProps);
		}
		if(!arePropsValid(intentProps)){
			//For the moment don't throw the execption throw new Exception("Props must be valid");
		}
		return props;
	}

	public boolean arePropsValid(Bundle intentBundle){
		boolean buildPropsEmpty = this.buildProps.isEmpty();
		boolean optionalPropsEmpty = this.optionalProps.isEmpty();
		boolean intentPropsEmpty = intentBundle.isEmpty();

		return buildPropsEmpty || intentPropsEmpty;
	}

	public String getVersionName() {
		return getProps().getString(TaConstants.TA_APP_VERSION_NAME);
	}

	public String[] getUniversalLinks() {
		String universalLinks = getProps().getString(TaConstants.TA_UNIVERSAL_LINKS);
		if (universalLinks != null) {
			return universalLinks.split(",");
		}
		return new String[0];
	}

	public String getUrlSchema() {
		return getProps().getString(TaConstants.TA_URL_SCHEMA);
	}


	/**
	 * if intent not contains machine and pSetup or an universal link
	 * means that the app is opened by click on drawer icon or is an helper app that has builtin pSetup and machine
	 * helper app (addon) contains psetup inside build
	 * drawer icon intent is empty
	 */
	public boolean intentNotEmpty(Intent intent){
		boolean isEmpty = true;
		if(intent != null){
			Uri universalLink = intent.getData();
			if (universalLink != null && Arrays.asList(getUniversalLinks()).contains(universalLink.getHost())) {
				isEmpty = false;
			} else if (intent.getExtras() != null) {
				Bundle extras = intent.getExtras();
				// use current pSetup and machine
				if (extras.containsKey(TaConstants.TA_PATH) && !extras.getString(TaConstants.TA_PATH).isEmpty()) {
					isEmpty = false;
				} else {
					// if intent contains pSetup and machine the app is opened by url, or from another app
					isEmpty = !extras.containsKey(TaConstants.TA_MACHINE) ||
							extras.getString(TaConstants.TA_MACHINE, "").isEmpty() ||
							!extras.containsKey(TaConstants.TA_PSETUP) || extras.getString(TaConstants.TA_PSETUP, "").isEmpty();
				}
			}
		}
		Log.d(TAG, "intent is empty: " + isEmpty);
		return !isEmpty;
	}

	public void setBuildProps(Bundle buildProps) {
		this.buildProps = buildProps;
	}

	public void setOptionalProps(Bundle optionalProps) {
		this.optionalProps = optionalProps;
	}

	private Bundle getProps(){
		Bundle props = new Bundle();
		if(buildProps != null) {
			props.putAll(buildProps);
		}
		if(optionalProps != null) {
			props.putAll(optionalProps);
		}
		try {
			props.putDouble(TaConstants.TA_DPI, TaUtils.getDensityDpi(mContext));
			props.putBoolean(TaConstants.TA_IS_KINDLE, TaUtils.isKindle());
			props.putBoolean(TaConstants.TA_IS_HUAWEI, TaUtils.isHuawei());
			props.putString(TaConstants.TA_WEB_PACKAGE, TaUtils.getSystemWebView(mContext));
		} catch(Exception e){
			e.printStackTrace();
		}
		return props;
	}

	private Bundle getBundleFromIntent(Intent intent){
		Bundle bundle = new Bundle();
		if(intent != null){
			Uri universalLink = intent.getData();
			if (universalLink != null && Arrays.asList(getUniversalLinks()).contains(universalLink.getHost())) {
				bundle.putString(TaConstants.TA_URL, universalLink.toString());
			} else if(intent.getExtras() != null && intent.getExtras().size() > 0){
				Bundle extras = intent.getExtras();
				String path = extras.getString(TaConstants.TA_PATH, "load");
				StringBuilder url = new StringBuilder(getUrlSchema() + "://" + path + "/?");
				for(String key : extras.keySet()){
					if(!excludeKey(key)) {
						url.append(key).append("=").append(extras.get(key)).append("&");
					}
				}
				bundle.putString(TaConstants.TA_URL, url.toString());
			}
		}
		return bundle;
	}

	private boolean excludeKey(String key){
		return key.startsWith("org.chromium") ||
				key.startsWith("android.intent") ||
				key.startsWith("com.android.browser");
	}
}
