package com.tecnavia.tabridge;

import android.app.Application;
import android.content.Context;
import android.content.SharedPreferences;
import android.util.Log;

import androidx.annotation.Nullable;
import com.facebook.react.ReactNativeHost;
import com.facebook.react.ReactPackage;
import com.facebook.react.bridge.JavaScriptExecutorFactory;
import com.facebook.hermes.reactexecutor.HermesExecutorFactory;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * must be static because mHost should be initialized just one time
 * otherwise the app crash
 * the crash occurs occurs when host is initiliazed inside activity delegate instead of application class
 */
public class TaReactNative {
	private static final String TAG = "TA_RN_LOADER";

	private static final String SP_NAME = "sp_main";
	private static final String SP_BUNDLE_NAME = "bundle_name";
	private static final String SP_BUNDLE_APP_VERSION = "bundle_app_version";
	private static TaReactNative instance;
	private ReactNativeHost mHost;

	private TaReactNative(){}

	public static TaReactNative getInstance(){
		if(instance == null){
			instance = new TaReactNative();
		}
		return instance;
	}

	public ReactNativeHost getHost(Application application, String versionName){
		if(mHost == null){
			mHost = new ReactNativeHost(application) {
				@Override
				public boolean getUseDeveloperSupport() {
					return BuildConfig.DEBUG;
				}

				@Override
				protected List<ReactPackage> getPackages() {
					List<ReactPackage> packages = new ArrayList<>();
					try {
						String[] assets = application.getAssets().list("");
						for (String asset : assets) {
							Pattern pattern = Pattern.compile("packages(_.*?)?\\.txt", Pattern.CASE_INSENSITIVE);
							Matcher matcher = pattern.matcher(asset);
							boolean matchFound = matcher.find();
							if (matchFound) {
								Log.d(TAG, "Loading packages from " + asset);
								BufferedReader br = null;
								try {
									InputStreamReader is = new InputStreamReader(application.getAssets().open(asset));
									br = new BufferedReader(is);
									String mLine;
									while ((mLine = br.readLine()) != null) {
										try {
											Log.d(TAG, "Adding react package " + mLine);
											packages.add((ReactPackage) (Class.forName(mLine).newInstance()));
										} catch (IllegalAccessException | InstantiationException |
												 ClassNotFoundException e) {
											Log.w(TAG, e.getMessage());
											e.printStackTrace();
										}
									}
								} catch (IOException e) {
									e.printStackTrace();
								} finally {
									if (br != null) {
										try {
											br.close();
										} catch (IOException e) {
											e.printStackTrace();
										}
									}
								}
							}
						}
					} catch (IOException e) {
						e.printStackTrace();
					}
					return packages;
				}

				@Nullable
				@Override
				protected String getJSBundleFile() {
					String bundleName = getBundleName(application.getApplicationContext(), versionName);
					if (BuildConfig.DEBUG || bundleName == null || bundleName.isEmpty()) {
						Log.d(TAG, "Use default bundle");
						return super.getJSBundleFile();
					}

					Log.d(TAG, "Use bundle " + bundleName);
					return bundleName;
				}

				@Override
				protected String getJSMainModuleName() {
					return "index";
				}

				@Nullable
				@Override
				protected JavaScriptExecutorFactory getJavaScriptExecutorFactory() {
					if (BuildConfig.ENABLE_HERMES) {
						return new HermesExecutorFactory();
					}
					return super.getJavaScriptExecutorFactory();
				}
			};
		}
		return mHost;
	}

	private String getBundleName(Context context, String versionName){
		//app version is different from bundle app version, so remove stored bundle and load default bundle
		if (versionName != null && !versionName.equals(TaReactNative.getBundleVersion(context))) {
			TaReactNative.setBundleName(context, "");
			return null;
		}
		String bundleName = TaReactNative.getBundleName(context);
		if (new File(bundleName).exists()) {
			return bundleName;
		}
		return null;
	}

	public static String getBundleName(Context reactApplicationContext) {
		SharedPreferences sp = reactApplicationContext.getSharedPreferences(SP_NAME, Context.MODE_PRIVATE);
		return sp.getString(SP_BUNDLE_NAME, "");
	}

	public static String getBundleVersion(Context reactApplicationContext) {
		SharedPreferences sp = reactApplicationContext.getSharedPreferences(SP_NAME, Context.MODE_PRIVATE);
		return sp.getString(SP_BUNDLE_APP_VERSION, "");
	}

	public static void setBundleName(Context context, String name){
		SharedPreferences sp = context.getSharedPreferences(SP_NAME, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = sp.edit();
		editor.putString(SP_BUNDLE_NAME, name);
		editor.apply();
	}

	public static void setBundleVersion(Context context, String version){
		SharedPreferences sp = context.getSharedPreferences(SP_NAME, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = sp.edit();
		editor.putString(SP_BUNDLE_APP_VERSION, version);
		editor.apply();
	}

	public static void destroy() {
		if (getInstance().mHost != null) {
			getInstance().mHost.clear();
			getInstance().mHost = null;
		}
		instance = null;
	}
}
