package com.tecnavia.tabridge.broadcasts;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;

import com.facebook.react.ReactInstanceManager;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;

public class TaReceiver extends BroadcastReceiver {
	public static final String BUNDLE_EXTRA = "data";
	public static final String BROADCAST_ON_NEW_INTENT = "com.tecnaviaapplication.newsmemory.onnewintent";
	public static final String BROADCAST_ON_KEY_PRESS_DOWN = "com.tecnaviaapplication.onkeyboardpress.down";
	public static final String BROADCAST_ON_KEY_PRESS_UP = "com.tecnaviaapplication.onkeyboardpress.up";

	public static final String BROADCAST_ON_USER_INTERACT = "com.tecnaviaapplication.onuserinteract";

	public static IntentFilter getBroadcastFilter(){
		IntentFilter filter = new IntentFilter();
		filter.addAction( BROADCAST_ON_NEW_INTENT );
		filter.addAction( BROADCAST_ON_KEY_PRESS_DOWN );
		filter.addAction( BROADCAST_ON_KEY_PRESS_UP );
		filter.addAction( BROADCAST_ON_USER_INTERACT );
		return filter;
	}

	private ReactInstanceManager instanceManager;

	public TaReceiver(ReactInstanceManager instanceManager) {
		this.instanceManager = instanceManager;
	}

	@Override
	public void onReceive(Context context, Intent intent) {
		String action = intent.getAction();
		if (action != null) {
			switch (action) {
				case BROADCAST_ON_NEW_INTENT:
					sendOnNewIntent(intent.getBundleExtra(BUNDLE_EXTRA));
					break;
				case BROADCAST_ON_KEY_PRESS_DOWN:
					sendOnKeyPressDown(intent.getBundleExtra(BUNDLE_EXTRA));
					break;
				case BROADCAST_ON_KEY_PRESS_UP:
					sendOnKeyPressUp(intent.getBundleExtra(BUNDLE_EXTRA));
					break;
				case BROADCAST_ON_USER_INTERACT:
					sendOnUserInteract();
			}
		}
	}

	public void sendOnKeyPressDown(Bundle data) {
		WritableMap bundle;
		try {
			bundle = Arguments.fromBundle(data);
		} catch (NullPointerException npe) {
			bundle = Arguments.createMap();
		}
		sendEvent(instanceManager, "onKeyDown", bundle);
	}

	public void sendOnUserInteract(){
		sendEvent(instanceManager, "onUserInteract", Arguments.createMap());
	}

	public void sendOnKeyPressUp(Bundle data) {
		WritableMap bundle;
		try {
			bundle = Arguments.fromBundle(data);
		} catch (NullPointerException npe) {
			bundle = Arguments.createMap();
		}
		sendEvent(instanceManager, "onKeyUp", bundle);
	}

	public void sendOnNewIntent(Bundle data) {
		WritableMap bundle;
		try {
			bundle = Arguments.fromBundle(data);
		} catch (NullPointerException npe) {
			bundle = Arguments.createMap();
		}
		sendEvent(instanceManager, "onNewIntent", bundle);
	}

	private void sendEvent(ReactInstanceManager instanceManager, String eventName, WritableMap map) {
		try {
			if (instanceManager != null && instanceManager.getCurrentReactContext() != null) {
				if (map == null) {
					map = Arguments.createMap();
				}
				instanceManager.getCurrentReactContext()
						.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
						.emit(eventName, map);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}