package com.tecnavia.tabridge.hacks;

import android.view.View;
import android.view.ViewGroup;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.ReadableType;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.uimanager.IllegalViewOperationException;
import com.facebook.react.uimanager.NativeViewHierarchyManager;
import com.facebook.react.uimanager.UIBlock;

/**
 * allow to customize native view attribute
 */
public class CustomNativeProps implements UIBlock {
	private static final String KEY_DESCENDANT_FOCUSABILITY = "descendantFocusability";
	private static final String KEY_FOCUSABLE_IN_TOUCH_MODE = "focusableInTouchMode";
	private static final String KEY_FOCUSABLE = "focusable";

	private static final String VALUE_AFTER_DESCENDANTS = "afterDescendants";
	private static final String VALUE_BEFORE_DESCENDANTS = "beforeDescendants";
	private static final String VALUE_BLOCK_DESCENDANTS = "blockDescendants";

	private final ReadableMap map;
	private final WritableMap error;
	private final Promise promise;

	public CustomNativeProps(ReadableMap map, Promise promise){
		this.map = map;
		this.promise = promise;
		this.error = Arguments.createMap();
	}

	private boolean checkingViewInstance(View view, String key){
		if (!(view instanceof ViewGroup)) {
			error.putString(key, "view is not a view group");
			return false;
		}
		return true;
	}

	private void setDescendantFocusability(View view){
		try {
			if (map.hasKey(KEY_DESCENDANT_FOCUSABILITY) && checkingViewInstance(view, KEY_DESCENDANT_FOCUSABILITY)) {
				String value = map.getString(KEY_DESCENDANT_FOCUSABILITY);
				if (VALUE_AFTER_DESCENDANTS.equals(value)) {
					((ViewGroup) view).setDescendantFocusability(ViewGroup.FOCUS_AFTER_DESCENDANTS);
				} else if (VALUE_BEFORE_DESCENDANTS.equals(value)) {
					((ViewGroup) view).setDescendantFocusability(ViewGroup.FOCUS_BEFORE_DESCENDANTS);
				} else if (VALUE_BLOCK_DESCENDANTS.equals(value)) {
					((ViewGroup) view).setDescendantFocusability(ViewGroup.FOCUS_BLOCK_DESCENDANTS);
				}
			}
		} catch (Exception e){
			e.printStackTrace();
			error.putString(KEY_DESCENDANT_FOCUSABILITY, e.getMessage());
		}
	}

	private void setFocusableInTouchMode(View view){
		try {
			if (map.hasKey(KEY_FOCUSABLE_IN_TOUCH_MODE)) {
				view.setFocusableInTouchMode(map.getBoolean(KEY_FOCUSABLE_IN_TOUCH_MODE));
			}
		} catch(Exception e){
			e.printStackTrace();
			error.putString(KEY_FOCUSABLE_IN_TOUCH_MODE, e.getMessage());
		}

	}

	private void setFocusable(View view){
		try {
			if (map.hasKey(KEY_FOCUSABLE)) {
				view.setFocusable(map.getBoolean(KEY_FOCUSABLE));
			}
		} catch(Exception e) {
			e.printStackTrace();
			error.putString(KEY_FOCUSABLE, e.getMessage());
		}
	}

	@Override
	public void execute(NativeViewHierarchyManager nativeViewHierarchyManager) {
		if(map == null){
			promise.reject(new Throwable("missing map"));
			return;
		}
		if(!map.hasKey("tag")){
			promise.reject(new Throwable("missing tag id"));
			return;
		}
		if(map.getType("tag") != ReadableType.Number){
			promise.reject(new Throwable("tag id must be integer"));
			return;
		}

		int tag = map.getInt("tag");
		try {
			View view = nativeViewHierarchyManager.resolveView(tag);
			if (view == null) {
				promise.reject(new Throwable("view not found"));
				return;
			}

			setDescendantFocusability(view);
			setFocusableInTouchMode(view);
			setFocusable(view);

			promise.resolve(error);
		} catch (IllegalViewOperationException e){
			promise.reject(e);
		}
	}
}