package com.tecnavia.tabridge.hacks;

import android.annotation.SuppressLint;
import android.view.View;
import android.view.ViewGroup;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.ReadableType;
import com.facebook.react.uimanager.IllegalViewOperationException;
import com.facebook.react.uimanager.NativeViewHierarchyManager;
import com.facebook.react.uimanager.UIBlock;

/**
 * allow to customize native view attribute
 */
public class CustomTouch implements UIBlock {
	private ReadableMap map;
	private Promise promise;

	public CustomTouch(ReadableMap map, Promise promise){
		this.map = map;
		this.promise = promise;
	}

	@SuppressLint("ClickableViewAccessibility")
	@Override
	public void execute(NativeViewHierarchyManager nativeViewHierarchyManager) {
		if(map == null){
			promise.reject(new Throwable("missing map"));
			return;
		}
		if(!map.hasKey("tag")){
			promise.reject(new Throwable("missing tag id"));
			return;
		}
		if(map.getType("tag") != ReadableType.Number){
			promise.reject(new Throwable("tag id must be integer"));
			return;
		}

		int tag = map.getInt("tag");
		try {
			View view = nativeViewHierarchyManager.resolveView(tag);
			if (view == null) {
				promise.reject(new Throwable("view not found"));
				return;
			}
			if (!(view instanceof ViewGroup)) {
				promise.reject(new Throwable("view is not a view group"));
				return;
			}

			view.setOnTouchListener((v, event) -> {
				int minPointers = -1;
				if(map.hasKey("minPointers")){
					minPointers = map.getInt("minPointers");
				}

				boolean result;
				if(event.getPointerCount() >= minPointers){
					result = view.onTouchEvent(event);
				} else {
					result = false;
				}
				return result;
			});
			promise.resolve(Arguments.createMap());
		} catch (IllegalViewOperationException e){
			promise.reject(e);
		}
	}
}