package com.tecnavia.tabridge.helpers;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.content.Context;
import android.os.Build;
import android.util.Log;

import androidx.annotation.RequiresApi;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReadableMap;
import com.tecnavia.tabridge.tasks.NotificationTask;

public class NotificationHelper {
	private static final String TAG = "Notification";

	@RequiresApi( api = Build.VERSION_CODES.O )
	public static void setupChannel(ReactApplicationContext reactContext, ReadableMap config, Promise promise){
		Log.d(TAG, "setup channel...");
		if(config == null){
			Log.d(TAG, "config is null");
			promise.reject(new Exception("configuration mustn't be empty"));
		} else if(!config.hasKey("id")) {
			Log.d(TAG, "missing channel id");
			promise.reject(new Exception("missing id"));
		} else {
			NotificationManager notificationManager = (NotificationManager) reactContext.getSystemService(Context.NOTIFICATION_SERVICE);
			String id = config.getString("id");
			String name = config.getString("name");
			String description = config.getString("description");
			int importance = config.hasKey("importance") ? config.getInt("importance") : NotificationManager.IMPORTANCE_DEFAULT;
			Log.d(TAG, "channel id: " + id);
			Log.d(TAG, "channel name: " + name);
			Log.d(TAG, "channel description: " + description);
			Log.d(TAG, "channel importance: " + importance);
			NotificationChannel channel = notificationManager.getNotificationChannel(id);
			if (channel == null) {
				try {
					channel = new NotificationChannel(id, name, importance);
					channel.enableLights(true);
					channel.setShowBadge(true);
					channel.enableVibration(importance >= NotificationManager.IMPORTANCE_LOW);
					channel.setDescription(description);
					channel.setLockscreenVisibility(Notification.VISIBILITY_PUBLIC);
					notificationManager.createNotificationChannel(channel);
					promise.resolve("created: " + id);
					Log.d(TAG, "channel created");
				} catch (Exception e) {
					promise.reject(e);
				}
			} else {
				Log.d(TAG, "channel already exists");
				promise.resolve(id + " already exists");
			}
		}
	}

	public static void removeDeliveredNotification(ReactApplicationContext reactContext, String notificationId, Promise promise) {
		try {
			Log.d(TAG, "remove notification id: " + notificationId);
			NotificationManager notificationManager = (NotificationManager) reactContext.getSystemService(Context.NOTIFICATION_SERVICE);
			notificationManager.cancel(notificationId.hashCode());
			promise.resolve(null);
		} catch (Exception e){
			promise.reject(e);
		}
	}

	public static void displayNotification(ReactApplicationContext reactContext, ReadableMap notification, Promise promise) {
		if(notification != null) {
			Log.d(TAG, "display notification: " + notification);
		}
		NotificationManager notificationManager = (NotificationManager) reactContext.getSystemService(Context.NOTIFICATION_SERVICE);
		new NotificationTask(notificationManager, Arguments.toBundle(notification), promise).execute(reactContext);
	}
}
