package com.tecnavia.tabridge.modules;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.SystemClock;
import android.provider.Settings;
import android.util.Base64;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.module.annotations.ReactModule;
import com.facebook.react.uimanager.UIManagerModule;
import com.tecnavia.tabridge.TaReactNative;
import com.tecnavia.tabridge.hacks.CustomNativeProps;
import com.tecnavia.tabridge.hacks.CustomTouch;
import com.tecnavia.tabridge.listeners.ActivityStarterModuleListener;
import com.tecnavia.tabridge.utils.TaAssets;
import com.tecnavia.tabridge.utils.TaFile;
import com.tecnavia.tabridge.utils.TaReflection;
import com.tecnavia.tabridge.utils.TaUtils;

import java.io.ByteArrayOutputStream;
import java.io.File;

@ReactModule(name  = "ActivityStarter")
public class ActivityStarterModule extends ReactContextBaseJavaModule {
    private ActivityStarterModuleListener listener;

    public ActivityStarterModule(ReactApplicationContext reactContext) {
        super(reactContext);
    }

    @Override
    public String getName() {
        return "ActivityStarter";
    }

    public void setListener(ActivityStarterModuleListener listener) {
        this.listener = listener;
    }

    @ReactMethod
    public void startNewsmemory( ReadableMap config, Promise promise){
        promise.reject(new NullPointerException("NewsMemory was removed"));
    }

    @ReactMethod
    public void closeNativeNewsMemory(Promise promise) {
        promise.reject(new NullPointerException("NewsMemory was removed"));
    }

    @ReactMethod
    public void initializeGlobals(Promise promise){
        if(listener != null) {
            promise.resolve(Arguments.fromBundle(listener.getInitProps()));
        } else {
            Bundle buildProps = TaReflection.getBuildConfigProps();
            Bundle optionalProps = TaReflection.getOptionalConfigProps();
            buildProps.putAll(optionalProps);
            promise.resolve(Arguments.fromBundle(buildProps));
        }
    }

    @ReactMethod
    public void isKindle(Promise promise){
        if(TaUtils.isKindle()){
            promise.resolve("Is kindle");
        } else {
            promise.reject(getName(), "Is not kindle");
        }
    }

    @ReactMethod
    public void saveBundleName(String bundleName, String bundleAppVersion, Promise promise){
        try {
            ReactApplicationContext context = getReactApplicationContext();
            TaReactNative.setBundleName(context, bundleName);
            TaReactNative.setBundleVersion(context, bundleAppVersion);
            promise.resolve(Arguments.createMap());
        } catch (Exception e){
            promise.reject(e);
        }
    }

    @ReactMethod
    public void getBundleName(Promise promise){
        try {
            ReactApplicationContext context = getReactApplicationContext();
            String bundleName = TaReactNative.getBundleName(context);
            String bundleVersion = TaReactNative.getBundleVersion(context);

            WritableMap map = Arguments.createMap();
            map.putString("bundleName", bundleName);
            map.putString("bundleAppVersion", bundleVersion);

            promise.resolve(map);
        } catch (Exception e){
            promise.reject(e);
        }
    }

    @ReactMethod
    public void restartReactContext(Promise promise){
        try {
            if(listener != null){
                listener.onReload();
                promise.resolve(Arguments.createMap());
            } else {
                promise.reject(new NullPointerException());
            }
        } catch (Exception e){
            promise.reject(e);
        }
    }

    @ReactMethod
    public void isActionModeVisible(Promise promise){
        try {
            WritableMap map = Arguments.createMap();
            if(listener != null){
                map.putBoolean("result", listener.isActionModeVisible());
            } else {
                map.putBoolean("result", false);
            }
            promise.resolve(map);
        } catch (Exception e){
            promise.reject(e);
        }
    }

    @ReactMethod
    public void getBase64FromRes(String assetName, Promise promise){
        try {
            int resourceId = getReactApplicationContext().getResources().getIdentifier(assetName, "drawable", getReactApplicationContext().getPackageName());
            // int resourceId = R.drawable.images_png_marker
            // This will work as well
            Bitmap image = BitmapFactory.decodeResource(getReactApplicationContext().getResources(), resourceId);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            image.compress(Bitmap.CompressFormat.PNG, 100, baos); // bm is the bitmap object
            String encoded = Base64.encodeToString(baos.toByteArray(), Base64.NO_WRAP);
            Log.d("BASE64", encoded);
            promise.resolve(encoded);
        } catch (Exception e){
            e.printStackTrace();
            promise.reject(e);
        }
    }

    @ReactMethod
    public void assetsCopy(String src, String destination, Promise promise){
        try {
            if(!TaFile.exists(destination + File.separator + src)) {
                TaAssets.copyFileOrDir(getReactApplicationContext(), src, destination);
            }
            promise.resolve("done");
        } catch (Exception e) {
            TaFile.deleteFolder(destination + File.separator + src);
            e.printStackTrace();
            promise.reject(e);
        }
    }

    @ReactMethod
    public void isOrientationLocked(Promise promise) {
        try{
            WritableMap result = Arguments.createMap();
            result.putBoolean("locked", Settings.System.getInt(getReactApplicationContext().getContentResolver(), Settings.System.ACCELEROMETER_ROTATION, 0) == 0);
            String orientation = "PORTRAIT";
            if(getReactApplicationContext().getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE){
                orientation = "LANDSCAPE";
            }
            result.putString("orientation",orientation);
            promise.resolve(result);
        } catch(Exception e) {
            promise.reject(e);
        }
    }

    /**
     * used from webview when it is inside a pan gesture to trigger touch
     * @param viewTag
     * @param x
     * @param y
     * @param promise
     */
    @ReactMethod
    public void dispatchTap(int viewTag, float x, float y, Promise promise){
        try {
            final UIManagerModule uiManager = getReactApplicationContext().getNativeModule(UIManagerModule.class);
            uiManager.addUIBlock(nativeViewHierarchyManager -> {
                try {
                    View view = nativeViewHierarchyManager.resolveView(viewTag);
                    // List of meta states found here: developer.android.com/reference/android/view/KeyEvent.html#getMetaState()
                    MotionEvent me;
                    me = MotionEvent.obtain(SystemClock.uptimeMillis(), SystemClock.uptimeMillis() + 50, MotionEvent.ACTION_DOWN, x, y, 0);
                    view.dispatchTouchEvent(me);
                    me = MotionEvent.obtain(SystemClock.uptimeMillis() + 100, SystemClock.uptimeMillis() + 150, MotionEvent.ACTION_UP, x, y, 0);
                    view.dispatchTouchEvent(me);
                    promise.resolve("dispatched");
                } catch (Exception e){
                    e.printStackTrace();
                    promise.reject(e);
                }
            });
        } catch (Exception e){
            e.printStackTrace();
            promise.reject(e);
        }
    }

    @ReactMethod
    public void setCustomNativeProps(ReadableMap config, Promise promise){
        final UIManagerModule uiManager = getReactApplicationContext().getNativeModule(UIManagerModule.class);
        if(uiManager != null) {
            uiManager.addUIBlock(new CustomNativeProps(config, promise));
        } else {
            promise.reject(new Exception("UIManager is null"));
        }
    }

    @ReactMethod
    public void setCustomOnTouch(ReadableMap config, Promise promise){
        final UIManagerModule uiManager = getReactApplicationContext().getNativeModule(UIManagerModule.class);
        if(uiManager != null) {
            uiManager.addUIBlock(new CustomTouch(config, promise));
        } else {
            promise.reject(new Exception("UIManager is null"));
        }
    }
    @ReactMethod
    public void onClose(Promise promise){
        boolean consumed = false;
        try {
            if (listener != null) {
                consumed = listener.handleClose();
            }
        } catch(Exception e){
            e.printStackTrace();
        }
        promise.resolve(consumed);
    }

    @ReactMethod
    public void willOpenURL(String url, Promise promise){
        boolean consumed = false;
        try {
            if (listener != null) {
                consumed = listener.handleOpenUrl(url);
            }
        } catch(Exception e){
            e.printStackTrace();
        }
        promise.resolve(consumed);
    }

    @ReactMethod
    public void onTrackAction(ReadableMap data, Promise promise){
        boolean consumed = false;
        try {
            if (listener != null) {
                consumed = listener.handleTrackAction(Arguments.toBundle(data));
            }
        } catch(Exception e){
            e.printStackTrace();
        }
        promise.resolve(consumed);
    }

    @ReactMethod
    public void onTokenExpired(Promise promise){
        boolean consumed = false;
        try {
            if (listener != null) {
                consumed = listener.handleTokenExpired();
            }
        } catch(Exception e){
            e.printStackTrace();
        }
        promise.resolve(consumed);
    }
}
