package com.tecnavia.tabridge.modules;

import android.content.Intent;
import android.os.Build;
import android.os.Bundle;

import androidx.core.app.RemoteInput;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.module.annotations.ReactModule;
import com.tecnavia.tabridge.helpers.NotificationHelper;

@ReactModule(name  = "Notifications")
public class NotificationsModule extends ReactContextBaseJavaModule {
    public NotificationsModule(ReactApplicationContext reactContext) {
        super(reactContext);
    }

    @Override
    public String getName() {
        return "Notifications";
    }

    @ReactMethod
    public void removeDeliveredNotification(String notificationId, Promise promise) {
        try {
            NotificationHelper.removeDeliveredNotification(getReactApplicationContext(), notificationId, promise);
        } catch (NullPointerException e){
            promise.reject(e);
        }
    }

    @ReactMethod
    public void displayNotification(ReadableMap notification, Promise promise) {
        try {
            NotificationHelper.displayNotification(getReactApplicationContext(), notification, promise);
        } catch (NullPointerException e){
            promise.reject(e);
        }
    }

    @ReactMethod
    public void createChannel(ReadableMap config, Promise promise) {
        try {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                NotificationHelper.setupChannel(getReactApplicationContext(), config, promise);
            } else {
                promise.reject(new Exception("unsupported by system"));
            }
        } catch (NullPointerException e){
            promise.reject(e);
        }
    }

    private WritableMap parseIntentForNotification(Intent intent) {
        WritableMap notificationOpenMap = parseIntentForRemoteNotification(intent);
        if (notificationOpenMap == null) {
            notificationOpenMap = parseIntentForLocalNotification(intent);
        }
        return notificationOpenMap;
    }

    private WritableMap parseIntentForLocalNotification(Intent intent) {
        if (intent.getExtras() == null || !intent.hasExtra("notificationId")) {
            return null;
        }

        WritableMap notificationMap = Arguments.makeNativeMap(intent.getExtras());
        WritableMap notificationOpenMap = Arguments.createMap();
        notificationOpenMap.putString("action", intent.getAction());
        notificationOpenMap.putMap("notification", notificationMap);

        // Check for remote input results
        Bundle remoteInput = RemoteInput.getResultsFromIntent(intent);
        if (remoteInput != null) {
            notificationOpenMap.putMap("results", Arguments.makeNativeMap(remoteInput));
        }

        return notificationOpenMap;
    }

    private WritableMap parseIntentForRemoteNotification(Intent intent) {
        // Check if FCM data exists
        if (intent.getExtras() == null || !intent.hasExtra("google.message_id")) {
            return null;
        }

        Bundle extras = intent.getExtras();

        WritableMap notificationMap = Arguments.createMap();
        WritableMap dataMap = Arguments.createMap();

        for (String key : extras.keySet()) {
            if (key.equals("google.message_id")) {
                notificationMap.putString("notificationId", extras.getString(key));
            } else if (key.equals("collapse_key")
                    || key.equals("from")
                    || key.equals("google.sent_time")
                    || key.equals("google.ttl")
                    || key.equals("_fbSourceApplicationHasBeenSet")) {
                // ignore known unneeded fields
            } else {
                dataMap.putString(key, extras.getString(key));
            }
        }
        notificationMap.putMap("data", dataMap);

        WritableMap notificationOpenMap = Arguments.createMap();
        notificationOpenMap.putString("action", intent.getAction());
        notificationOpenMap.putMap("notification", notificationMap);

        return notificationOpenMap;
    }

    private WritableMap parseNotificationBundle(Bundle notification) {
        return Arguments.makeNativeMap(notification);
    }
}
