package com.tecnavia.tabridge.utils;

import android.content.Context;
import android.content.ContextWrapper;
import android.content.res.Resources;
import android.util.Log;

import java.util.Locale;

public class TaLang {
    public static ContextWrapper changeLang( Context c, String buildLanguage) {
        return updateResources(c, getLanguage(buildLanguage));
    }

    public static boolean isRTL(String buildLanguage) {
        return isLocaleRTL(getLanguage(buildLanguage));
    }

    private static boolean isLocaleRTL(Locale locale) {
        final int directionality = Character.getDirectionality(locale.getDisplayName().charAt(0));
        return directionality == Character.DIRECTIONALITY_RIGHT_TO_LEFT ||
                directionality == Character.DIRECTIONALITY_RIGHT_TO_LEFT_ARABIC;
    }

    private static ContextWrapper updateResources(Context context, Locale newLocale) {
        Locale.setDefault(newLocale);

        Resources rs = context.getResources();
        rs.getConfiguration().setLocale(newLocale);
        rs.updateConfiguration(rs.getConfiguration(), rs.getDisplayMetrics());

        //context = context.createConfigurationContext(config);
        return new ContextWrapper( context );
    }

    public static Locale getLanguage(String buildLanguage){
        Locale locale;
        Log.d( "UTILS LANG", "set language -> " + buildLanguage );
        switch( buildLanguage.toLowerCase( ) ){
            case "1":
            case "it":
            case "italian":
                locale = Locale.ITALY;
                break;
            case "2":
            case "de":
            case "german":
                locale = Locale.GERMANY;
                break;
            case "3":
            case "fr-fr":
            case "french":
                locale = Locale.FRANCE;
                break;
            case "4":
            case "nl-nl":
            case "dutch":
                locale = new Locale( "nl", "NL" );
                break;
            case "11":
            case "nl-be":
                locale = new Locale( "nl", "BE" );
                break;
            case "5":
            case "es-es":
            case "spanish":
                locale = new Locale( "spa", "ESP" );
                break;
            case "6":
            case "ro-ro":
            case "romanian":
                locale = new Locale( "ro", "RO" );
                break;
            case "7":
            case "swedish":
            case "sv":
                locale = new Locale( "sv", "SE" );
                break;
            case "8":
            case "danish":
            case "da":
                locale = new Locale( "da", "DK" );
                break;
            case "9":
            case "portuguese":
            case "pt-pt":
                locale = new Locale( "pt", "PT" );
                break;
            case "10":
            case "brasilian":
            case "pt-br":
                locale = new Locale( "pt", "BR" );
                break;
            case "12":
            case "norwey":
            case "nn-no":
                locale = new Locale( "nn", "NO" );
                break;
            case "13":
            case "nb-no":
                locale = new Locale( "nb", "NO" );
                break;
            case "14":
            case "arabic":
            case "ar":
                locale = new Locale( "ar", "" );
                break;
            default:
                try{
                    String country = Locale.getDefault( ).getDisplayCountry( );
                    switch( country ){
                        case "Australia":
                            locale = new Locale( "en", "AU" );
                            break;
                        case "United Kingdom":
                            locale = new Locale( "en", "GB" );
                            break;
                        default:
                            locale = new Locale( "en", "US" );
                            break;
                    }
                }
                catch( Exception e ){
                    locale = Locale.ENGLISH;
                }
                break;
        }
        return locale;
    }
}