package com.tecnavia.tabridge.utils;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;

import com.facebook.react.ReactInstanceManager;
import com.facebook.react.ReactNativeHost;
import com.facebook.react.ReactRootView;
import com.facebook.react.bridge.ReactApplicationContext;
import com.tecnavia.tabridge.BuildConfig;
import com.tecnavia.tabridge.TaApplication;
import com.tecnavia.tabridge.TaConstants;
import com.tecnavia.tabridge.TaReactNative;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Proxy;

public class TaReflection {
	private static final String TAG = "TA_REFLECTION";

    /**
	 * usefull if in you application there is a method getFileProviderAuthority used by cl.json.ShareApplication package
	 */
	public static void addShareInterface(TaApplication application){
		try {
			//Loading the class at runtime
			Class<?> shareInterface = Class.forName("cl.json.ShareApplication");
			Proxy.newProxyInstance(shareInterface.getClassLoader(), new Class<?>[]{shareInterface}, (proxy, method, args) -> {
				try {
					//Handle the invocations
					if (method.getName().equals("getFileProviderAuthority")) {
						return application.getFileProviderAuthority();
					}
				} catch (Exception e){
					e.printStackTrace();
				}
				return "";
			});
		} catch (ClassNotFoundException e) {
			Log.w(TAG, e.getMessage());
		}
	}

	/**
	 * Loads Flipper in React Native templates. Call this in the onCreate method with something like
	 * initializeFlipper(this, getReactNativeHost().getReactInstanceManager());
	 */
	public static void initializeFlipper(Context context, ReactInstanceManager reactInstanceManager) {
		try {
            /*
             We use reflection here to pick up the class that initializes Flipper,
            since Flipper library is not available in release mode
            */
			Class<?> aClass = Class.forName("com.tecnavia.tabridge.ReactNativeFlipper");
			aClass.getMethod("initializeFlipper", Context.class, ReactInstanceManager.class).invoke(null, context, reactInstanceManager);
		} catch (ClassNotFoundException | NoSuchMethodException | IllegalAccessException | InvocationTargetException | AssertionError e) {
			Log.w(TAG, e.getMessage());
		}
	}


	public static ReactRootView initRootView(String name, Context context){
		try {
			Class<?> cl = Class.forName(name);
			Constructor<?> cons = cl.getConstructor(Context.class);
			return (ReactRootView) cons.newInstance(context);
		} catch (IllegalAccessException | InstantiationException | ClassNotFoundException | NoSuchMethodException | InvocationTargetException e) {
			Log.w(TAG, e.getMessage());
		}
		return new ReactRootView(context);
	}

	public static void showSplashScreen(final Activity activity, String color) {
		try {
            /*
             We use reflection here to pick up the SplashScreen class,
            since SplashScreen library is optional
            */
			Class<?> aClass = Class.forName("org.devio.rn.splashscreen.SplashScreen");
			aClass.getMethod("show", Activity.class, String.class).invoke(null, activity, color);
		} catch (ClassNotFoundException | NoSuchMethodException | IllegalAccessException | InvocationTargetException | AssertionError e) {
			Log.w(TAG, e.getMessage());
		}
	}

	private static String getBuildField(String field){
		try {
			return (String) Class.forName("com.tecnaviaapplication.BuildConfig").getField(field).get(null);
		} catch (NoSuchFieldException | ClassNotFoundException | IllegalAccessException e) {
			e.printStackTrace();
		}
		return "";
	}

	public static Bundle getBuildConfigProps() {
		Bundle map = new Bundle();
		map.putString(TaConstants.TA_PSETUP, getBuildField("PSETUP"));
		map.putString(TaConstants.TA_MACHINE, getBuildField("SERVER"));
		map.putString(TaConstants.TA_LOCALE, getBuildField("LOCALE"));
		map.putString(TaConstants.TA_APP_VERSION_NAME, getBuildField("APP_VERSION_NAME"));
		map.putString(TaConstants.TA_APP_VERSION_CODE, getBuildField("APP_VERSION_CODE"));
		map.putString(TaConstants.TA_ANDROID_APP_ID, getBuildField("ANDROID_APP_ID"));
		map.putString(TaConstants.TA_APP_NAME, getBuildField("APP_NAME"));
		Log.d(TAG, map.toString());
		return map;
	}

	public static Bundle getOptionalConfigProps(){
		Bundle map = new Bundle();
		map.putString(TaConstants.TA_SPLASH_COLOR, getBuildField("SPLASH_BGCOLOR"));
		map.putString(TaConstants.TA_LOADING_COLOR, getBuildField("LOADING_COLOR"));
		map.putString(TaConstants.TA_URL_SCHEMA, getBuildField("ANDROID_UNIVERSAL_SCHEMA"));
		map.putString(TaConstants.TA_UNIVERSAL_LINKS, getBuildField("UNIVERSAL_LINKS"));
		map.putString(TaConstants.TA_ANDROID_VERSION_NAME, getBuildField("ANDROID_VERSION_NAME"));
		map.putString(TaConstants.TA_IOS_IAP_SHARED_SECRET, getBuildField("IOS_IAP_SHARED_SECRET"));
		map.putString(TaConstants.TA_PUSH_SERVICE, getBuildField("PUSH_SERVICE"));
		map.putString(TaConstants.TA_PUSH_CHANNEL, getBuildField("PUSH_CHANNEL"));
		map.putString(TaConstants.TA_PUSH_CHANNEL_SILENT, getBuildField("PUSH_CHANNEL_SILENT"));
		map.putString(TaConstants.TA_HCM_APP_ID, getBuildField("HUAWEI_APP_ID"));
		map.putString(TaConstants.TA_ONE_SIGNAL_APP_ID, getBuildField("ONE_SIGNAL_APP_ID"));
		map.putString(TaConstants.TA_MAS_APP_ID, getBuildField("MAS_APP_ID"));
		map.putString(TaConstants.TA_FACEBOOK_APP_ID, getBuildField("FACEBOOK_APP_ID"));
		map.putString(TaConstants.TA_API_KEY, getBuildField("API_KEY"));
		map.putBoolean(TaConstants.TA_IS_ADDON, false);
		map.putBoolean(TaConstants.TA_DEBUG_MODE, BuildConfig.DEBUG || getBuildField("DEBUG_MODE").equals("1"));
		Log.d(TAG, map.toString());
		return map;
	}

	public static ReactInstanceManager getReactInstanceManager(ReactApplicationContext reactContext) {
		String versionName = getBuildField("ANDROID_VERSION_NAME");
		ReactNativeHost mHost = TaReactNative.getInstance().getHost((Application) reactContext.getApplicationContext(), versionName);
		return mHost.getReactInstanceManager();
	}
}
