package com.tecnavia.tabridge.utils;

import android.app.ActivityManager;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.os.Build;
import android.util.DisplayMetrics;
import android.util.Log;

import androidx.webkit.WebViewCompat;

import java.lang.reflect.Method;

public class TaUtils {
    public static int getDensityDpi(Context context){
        try {
            return context.getResources().getDisplayMetrics().densityDpi;
        } catch (Exception e){
            e.printStackTrace();
        }
        return DisplayMetrics.DENSITY_DEFAULT;
    }


    public static boolean isKindle() {
        String AMAZON = "Amazon";
        String KINDLE = "Kindle";
        String FIRE = "Fire";

        if (Build.MANUFACTURER.contains(AMAZON) || (Build.MODEL.contains(KINDLE) && Build.MODEL.contains(FIRE))
                || Build.MODEL.startsWith("KF")) {
            return true;
        }
        try {
            Class.forName("com.amazon.device.messaging.ADM");
            return true;
        } catch (ClassNotFoundException | LinkageError e) {
            return false;
        }
    }

    public static boolean isHuawei(){
        return Build.MANUFACTURER.contains("Huawei");
    }

    public static String getSystemWebView(Context context) {
        try {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                PackageInfo info = WebViewCompat.getCurrentWebViewPackage(context);
                if(info != null) {
                    return info.packageName;
                }
            }
            Class<?> webViewFactory = Class.forName("android.webkit.WebViewFactory");
            Method method = webViewFactory.getMethod("getLoadedPackageInfo");
            PackageInfo packageInfo = (PackageInfo) method.invoke(null, null);
            if(packageInfo != null) {
                return packageInfo.packageName;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return "";
    }

    public static String getProcessName(Context context) {
        if (context == null) {
            return null;
        }
        ActivityManager manager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        int myPid = android.os.Process.myPid();
        Log.d("APP_UTILS", "my pid: " + myPid);
        for (ActivityManager.RunningAppProcessInfo processInfo : manager.getRunningAppProcesses()) {
            if (processInfo.pid == android.os.Process.myPid()) {
                Log.d("APP_UTILS", "process name: " + processInfo.processName);
                return processInfo.processName;
            }
        }
        return null;
    }
}