//
//  ActivityStarter.m
//  TecnaviaApplication
//
//  Created by iPad Developer on 10.01.19.
//  Copyright © 2019 Facebook. All rights reserved.
//

#import "ActivityStarter.h"
#import <React/RCTLog.h>
#import "BridgeEmitter.h"
#import "ReactBridgeManager.h"
#import <React/RCTUIManager.h>
#import <React/RCTView.h>

@implementation ActivityStarter{
    BridgeEmitter *emitter;
}

RCT_EXPORT_MODULE();

+ (BOOL)requiresMainQueueSetup
{
  return YES;
}

RCT_REMAP_METHOD(saveBundleName, bundleName:(NSString *)name bundleAppVersion:(NSString *)version saveBundleNameResolve:(RCTPromiseResolveBlock)resolve saveBundleNameReject:(RCTPromiseRejectBlock)reject)
{
    RCTLogInfo(@"saveBundleName:%@ version:%@", name, version);
      
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
    NSString *cacheDirectory = [[paths objectAtIndex:0] stringByAppendingString:@"/"];
    //make it absolute path as the application root folder name can change
    NSString *relativeName = [name stringByReplacingOccurrencesOfString:cacheDirectory withString:@""];
      
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:relativeName forKey:@"react_bundle_name"];
    [userDefaults setObject:version forKey:@"react_bundle_version"];
    [userDefaults synchronize];
    resolve(nil);
}

RCT_REMAP_METHOD(getBundleName, getBundleNameResolve:(RCTPromiseResolveBlock)resolve getBundleNameReject:(RCTPromiseRejectBlock)reject)
{
    RCTLogInfo(@"getBundleName");
    
    NSString *name = @"";
    NSString *version = @"";
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    NSString *bundleName = [userDefaults stringForKey:@"react_bundle_name"];
    if (bundleName != nil && [bundleName length] > 0) {
        NSArray *paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
        NSString *cacheDirectory = [paths objectAtIndex:0];
        
        name = [cacheDirectory stringByAppendingPathComponent:bundleName];
    }
    if ([userDefaults stringForKey:@"react_bundle_version"] != nil) {
        version = [userDefaults stringForKey:@"react_bundle_version"];
    }
    resolve(@{@"bundleName": name, @"bundleAppVersion": version});
}

RCT_REMAP_METHOD(restartReactContext, restartResolve:(RCTPromiseResolveBlock)resolve restartReject:(RCTPromiseRejectBlock)reject)
{
    RCTLogInfo(@"restartReactContext");
    dispatch_async(dispatch_get_main_queue(), ^{
        ReactBridgeManager *bridgeManager = [ReactBridgeManager sharedManager];
        [bridgeManager reload];
    });
    resolve(nil);
}

RCT_REMAP_METHOD(getStatusBarOrientation, orientationResolve:(RCTPromiseResolveBlock)resolve orientationReject:(RCTPromiseRejectBlock)reject)
{
  dispatch_async(dispatch_get_main_queue(), ^{
    UIInterfaceOrientation orientation = [[UIApplication sharedApplication] statusBarOrientation];

    NSString *orientationString = @"PORTRAIT";
    if(orientation == UIInterfaceOrientationLandscapeRight) {
      orientationString = @"LANDSCAPERIGHT";
    } else if (orientation ==  UIInterfaceOrientationLandscapeLeft){
      orientationString = @"LANDSCAPELEFT";
    } else if (orientation ==  UIInterfaceOrientationPortraitUpsideDown) {
      orientationString = @"PORTRAITUPSIDEDOWN";
    }
    resolve(orientationString);
  });
}

@end
