//
//  BridgeEmitter.m
//  TecnaviaApplication
//
//  Created by iPad Developer on 12.02.19.
//  Copyright © 2019 Facebook. All rights reserved.
//

#import <React/RCTLog.h>
#import "BridgeEmitter.h"

@implementation BridgeEmitter
{
  BOOL hasListeners;
}

RCT_EXPORT_MODULE();

+ (id)allocWithZone:(NSZone *)zone {
  static BridgeEmitter *sharedInstance = nil;
  static dispatch_once_t onceToken;
  dispatch_once(&onceToken, ^{
    sharedInstance = [super allocWithZone:zone];
  });
  return sharedInstance;
}

- (void)sendKeyDown:(NSDictionary *)event {
  [self sendEvent:@"onKeyDown" withBody:event];
}

- (void)sendKeyUp:(NSDictionary *)event {
  [self sendEvent:@"onKeyUp" withBody:event];
}

// Will be called when this module's first listener is added.
-(void)startObserving {
  hasListeners = YES;
}

// Will be called when this module's last listener is removed, or on dealloc.
-(void)stopObserving {
  hasListeners = NO;
}

- (void)sendEvent:(NSString *)event {
  [self sendEvent:event withBody:@{}];
}

- (void)sendEvent:(NSString *)event withBody:(NSDictionary *)body {
  if (hasListeners) {
    [self sendEventWithName:event body:body];
  }
}

- (NSArray<NSString *> *)supportedEvents {
  return @[@"onNewIntent", @"onKeyDown", @"onKeyUp"];
}

@end
