//
//  ReactBridgeManager.m
//  TecnaviaApplication
//
//  Created by iPad Developer on 18.01.21.
//  Copyright © 2021 Facebook. All rights reserved.
//

#import "ReactBridgeManager.h"
#import <React/RCTBundleURLProvider.h>
#import <React/RCTReloadCommand.h>

@interface ReactBridgeManager () <RCTBridgeDelegate>
@property (nonatomic, strong, readwrite) RCTBridge *bridge;
@end

@implementation ReactBridgeManager

+ (id)sharedManager {
    static ReactBridgeManager *sharedReactBridgeManager = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedReactBridgeManager = [[self alloc] init];
    });
    return sharedReactBridgeManager;
}

- (void) initBridgeWithLaunchOptions:(NSDictionary *)launchOptions
{
  if (self)
  {
    self.bridge = [[RCTBridge alloc] initWithDelegate:self launchOptions:launchOptions];
  }
}

- (void) reload {
    //[self.bridge reload];
    RCTTriggerReloadCommandListeners(@"react native bundle update");
}

#pragma mark - RCTBridgeDelegate methods

- (NSURL *)sourceURLForBridge:(RCTBridge *)bridge
{
#ifdef TA_USE_OLD_RN
    NSURL *jsCodeLocation = [[RCTBundleURLProvider sharedSettings] jsBundleURLForBundleRoot:@"index" fallbackResource:nil];
#else
    NSURL *jsCodeLocation = [[RCTBundleURLProvider sharedSettings] jsBundleURLForBundleRoot:@"index"];
#endif
    // Get the application sandbox root directory
    NSString *relativeBundleName = [[NSUserDefaults standardUserDefaults] stringForKey:@"react_bundle_name"];
    if (relativeBundleName != nil && [relativeBundleName length] > 0) {
        NSArray *paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
        NSString *cacheDirectory = [paths objectAtIndex:0];
        
        NSString *bundleName = [cacheDirectory stringByAppendingPathComponent:relativeBundleName];
        if ([[NSFileManager defaultManager] fileExistsAtPath:bundleName]) {
            // Adjusted load path
            jsCodeLocation = [NSURL URLWithString:bundleName];
            NSLog(@"new jsCodeLocation = %@",jsCodeLocation);
        }
    }
    
    return jsCodeLocation;
}

@end
