//
//  ReactViewController.m
//  TecnaviaApplication
//
//  Created by iPad Developer on 18.01.21.
//  Copyright © 2021 Facebook. All rights reserved.
//

#import "ReactViewController.h"
#import <React/RCTRootView.h>
#import <React/RCTLog.h>
#import "BridgeEmitter.h"

NSString *const TAPSetupPropKey = @"pSetup";
NSString *const TAServerPropKey = @"machine";
NSString *const TALocalePropKey = @"language";
NSString *const TASplashBackgroundColorPropKey = @"splashBgColor";
NSString *const TALoadingColorPropKey = @"loadingColor";
NSString *const TAAppNamePropKey = @"appName";
NSString *const TAAppVersionNamePropKey = @"appVersionName";
NSString *const TAAppVersionCodePropKey = @"appVersionCode";
NSString *const TAIAPSharedSecretPropKey = @"iosIapSharedSecret";
NSString *const TAPushServicePropKey = @"pushService";
NSString *const TAPushChannelPropKey = @"pushChannel";
NSString *const TAPushChannelSilentPropKey = @"pushChannelSilent";
NSString *const TAOneSignalAppIdPropKey = @"oneSignalAppId";
NSString *const TAMasAppIdPropKey = @"masAppId";
NSString *const TAFacebookAppIdPropKey = @"facebookAppId";
NSString *const TADebugModePropKey = @"debugMode";
NSString *const TAAndroidAppIdPropKey = @"androidAppId";
NSString *const TAAmazonIapDeveloperSecretPropKey = @"amazonIapDeveloperSecret";
NSString *const TAInitialOrientationPropKey = @"initialOrientation";
NSString *const TADeveloperNamePropKey = @"developerName";
NSString *const TAExternalLinkAccountURLPropKey = @"externalLinkAccountURL";
NSString *const TAIsAddonPropKey = @"isAddon";
NSString *const TAAddonNeedsCloseButtonPropKey = @"addonNeedsCloseButton";
NSString *const TAAddonNeedsSafeAreaPropKey = @"addonNeedsSafeArea";
NSString *const TATokenPropKey = @"token";
NSString *const TAAPIKeyPropKey = @"apiKey";
NSString *const TALockedOrientationPropKey = @"lockedOrientation";
NSString *const TALockedOrientationDevicePropKey = @"lockedOrientationDevice";
NSString *const TAEnableDebuggerPropKey = @"enableDebugger";

@implementation ReactViewController{
    BridgeEmitter *emitter;
}

- (instancetype)initWithBridge:(RCTBridge *)bridge moduleName:(NSString*)moduleName initialProps:(NSDictionary*)initialProps
{
  self = [super init];
  if(self)
  {
    RCTRootView *rootView = [[RCTRootView alloc] initWithBridge:bridge moduleName:moduleName initialProperties:initialProps];
    rootView.backgroundColor = [[UIColor alloc] initWithRed:1.0f green:1.0f blue:1.0f alpha:1];
    
    self.view = rootView;
    
    if (self->emitter == nil){
      self->emitter = [BridgeEmitter allocWithZone:nil];
    }
  }
  return self;
}

- (instancetype)initBridgeWithLaunchOptions:(NSDictionary *)launchOptions moduleName:(NSString*)moduleName initialProps:(NSDictionary*)initialProps
{
    //when your app is initialized, create the bridge manager and hold a reference to it.
    ReactBridgeManager *bridgeManager = [ReactBridgeManager sharedManager];
    [bridgeManager initBridgeWithLaunchOptions:launchOptions];
    //create a view controller that's controlling a react root view
    return [self initWithBridge:bridgeManager.bridge moduleName:moduleName initialProps:initialProps];
}

- (NSDictionary *)getKeyEvent:(UIPress *)keyPressed {
  NSDictionary *event = @{};
  if (@available(iOS 13.4, *)) {
    UIKey *key = [keyPressed key];
    NSNumber *keyCode = [[NSNumber alloc] initWithLong:(long)[key keyCode]];
    NSNumber *timestamp = [[NSNumber alloc] initWithFloat:[keyPressed timestamp]];
    NSString *characters = [key characters];
    
    event = @{@"keyCode": keyCode, @"downTime": timestamp, @"characters": characters};
    
    RCTLogInfo(@"Key press event: %@",event);
  }
  
  return event;
}

- (void)pressesBegan:(NSSet<UIPress *> *)presses withEvent:(UIPressesEvent *)event {
  if (@available(iOS 13.4, *)) {
    NSArray<UIPress *> *keys = [presses allObjects];
    for (int i = 0; i < [keys count]; i++) {
      UIPress *keyPressed = keys[i];
      if (keyPressed != nil && [keyPressed key] != nil) {
          NSDictionary *keyEvent = [self getKeyEvent:keyPressed];
          RCTLogInfo(@"send key down event: %@", keyEvent);
          [self->emitter sendKeyDown:keyEvent];
      }
    }
  } else {
    // Fallback on earlier versions
    RCTLogInfo(@"Unsupported keyboard event");
  }
  [super pressesBegan:presses withEvent:event];
}

- (void)pressesEnded:(NSSet<UIPress *> *)presses withEvent:(UIPressesEvent *)event {
  if (@available(iOS 13.4, *)) {
    NSArray<UIPress *> *keys = [presses allObjects];
    for (int i = 0; i < [keys count]; i++) {
      UIPress *keyPressed = keys[i];
      if (keyPressed != nil && [keyPressed key] != nil) {
          NSDictionary *keyEvent = [self getKeyEvent:keyPressed];
          RCTLogInfo(@"send key up event: %@", keyEvent);
          [self->emitter sendKeyUp:keyEvent];
      }
    }
  } else {
    // Fallback on earlier versions
    RCTLogInfo(@"Unsupported keyboard event");
  }
  [super pressesEnded:presses withEvent:event];
}

- (void)pressesCancelled:(NSSet<UIPress *> *)presses withEvent:(UIPressesEvent *)event {
  if (@available(iOS 13.4, *)) {
    NSArray<UIPress *> *keys = [presses allObjects];
    for (int i = 0; i < [keys count]; i++) {
      UIPress *keyPressed = keys[i];
      if (keyPressed != nil && [keyPressed key] != nil) {
          NSDictionary *keyEvent = [self getKeyEvent:keyPressed];
          RCTLogInfo(@"send key up event: %@", keyEvent);
          [self->emitter sendKeyUp:keyEvent];
      }
    }
  } else {
    // Fallback on earlier versions
    RCTLogInfo(@"Unsupported keyboard event");
  }
  [super pressesCancelled:presses withEvent:event];
}

@end
